#ifndef C_P1_META_DATA_H
#define C_P1_META_DATA_H

#include "C_P1ImageCommonLinkage.h"
#include "C_P1ImageCommonErrorCodes.h"
#include "C_P1ImageCommonTypes.h"

#include <vector>
#include <string>

namespace P1
{    
namespace ImageSdkC
{
    // forward declarations
    typedef void* P1ImageSDK_CaptureImage;
    class CMetaDataDatabase;

    struct CURational
    {
        uint32_t nom, denom;
    };


    struct CSRational
    {
        int32_t nom, denom;
    };


    class P1MetaData
    {
    public:
        virtual uint32_t GetDataType() { return 0; }
    };

    class P1MetaObject
    {
    public:
        P1MetaObject(P1MetaData* p=nullptr) : mData(p) {}



        bool IsValid();


        uint32_t GetType() const;


        uint32_t GetCount() const;


        uint8_t GetUByte(bool& error);   


        uint16_t GetUWord(bool& error);


        uint32_t GetULong(bool& error);   


        uint8_t GetSByte(bool& error);  


        uint16_t GetSWord(bool& error);  


        uint32_t GetSLong(bool& error); 


        float GetFloat(bool& error);   


        double GetDouble(bool& error);  


        CURational GetURational(bool& error);


        CSRational GetSRational(bool& error);


        std::string GetRawString(bool& error);

 
        std::string GetString(bool& error);


        // Getters of arrays

        std::vector<uint8_t> GetUBytes(bool& error);

   
        std::vector<uint16_t> GetUWords(bool& error);

 
        std::vector<uint32_t> GetULongs(bool& error);


        std::vector<int8_t> GetSBytes(bool& error);


        std::vector<int16_t> GetSWords(bool& error);


        std::vector<int32_t> GetSLongs(bool& error);


        std::vector<float> GetFloats(bool& error);


        std::vector<double> GetDoubles(bool& error);        


        bool SetUByte(uint8_t value);


        bool SetUWord(uint16_t value);
        

        bool SetULong(uint32_t value);


        bool SetSByte(uint8_t value);

       
        bool SetSWord(uint16_t value);


        bool SetSLong(uint32_t value);


        bool SetFloat(float value);


        bool SetDouble(double value);


        bool SetURational(CURational value);


        bool SetSRational(CSRational value);


        bool SetString(std::string buffer);

 
        bool SetUBytes(uint8_t* values, uint32_t count);


        bool SetUWords(uint16_t* values, uint32_t count);


        bool SetULongs(uint32_t* values, uint32_t count);


        bool SetSBytes(int8_t* values, uint32_t count);


        bool SetSWords(int16_t* values, uint32_t count);


        bool SetSLongs(int32_t* values, uint32_t count);

 
        bool SetFloats(float* values, uint32_t count);


        bool SetDoubles(double* values, uint32_t count);
      

        bool IsReadOnly();


        bool IsWritable();


        void SetWritable(bool state);

  
        bool IsDisabled();


        void SetDisabled(bool state);


        bool IsModifiable();


        // Internal function, do not use
        P1MetaData* GetPtr() { return mData;  }

    private:
        P1MetaData* mData = nullptr;
    };
                 




    class EXPORT_API P1MetaDataBase
    {
    public:


        P1MetaDataBase(P1::ImageSdkC::P1ImageSDK_CaptureImage image, bool useDbFromFile = false);


        ~P1MetaDataBase();


        P1MetaObject GetMetaObject(std::string path);


        P1MetaObject CreateMetaObject(std::string path);
    

        bool Exist(std::string path);               


        bool CreateXmpNamespace(std::string name, std::string url);


        void Dump();
        

        CMetaDataDatabase* GetDb();

    private:
        CMetaDataDatabase* mDb = nullptr;
        bool releaseData = false;
    };

}
}



#endif
